#!/usr/bin/perl -w
# A perl script to handle comments on my web site
# V 0.1: "BreakMe"
# Copyright (c) 2007, Timothy Pollard
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
# 
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Allow the use of CGI.pm
use CGI ":standard";

use HTML::Entities;

# Get the configuration
sub get_conf
{
    # Uses hard coded values, but would be better with a configuration file
    $home_page       = "http:\/\/bur.st\/~timp\/";
    $page_name       = "Tim's Bits and Pieces";
    $com_div         = "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%";
    $webmaster_email = "timp\@bur.st";
    $css             = "../style.css";
}

# Prints the custom top of page info
sub print_top_of_page
{
    print "<h1><a href=\"$home_page\">$page_name</a></h1>\n";
    print "<p><a href=\"..\/$post\">Return to the original Post</a></p>\n";
}

# Executed if there is something wrong with the query string
sub print_query_error
{
    print start_html (-title=>"Bad Query", -style=>{'src'=>$css});
    print h1 ("Bad Query: $error_msg");
    print "<p>To see the comments for a post open the post and click on the ",
          "comments link. You can add comments from the resulting page.\n";
    print "<a href=\"$home_page\">Click here to return to the home page<\/a><\/p>";
;
}

# Get's and checks the name of the post
sub get_post
{
    $post = param ("post");
    if (-e "..\/$post") # The post directory exists
    {   return $post;   }
    else # The directory that should contain the post does not exist
    {   
        # Generate an error page and exit the script
        $error_msg = "Post Not Found";
        print p ("..\/$post");
        print_query_error;
        print end_html;
        exit 1;
    }
}

# Print an error if the comments file is corrupt
sub print_read_error
{
    print h4 ("An error was encountered when trying to read the comments file.
    Please email the <a href=\"mailto:\/\/$webmaster_email\">webmaster</a>
    listing the this pages url.");
}

# Read the comments
sub read_comments
{
    # Set the number of comments
    $num_comments = 0;

    open (COMFILE, "..\/$post\/comments") || print_read_error && return;

    @com_file = <COMFILE>;

    close (COMFILE);

    @comments = ([]);

    $end_loop = 0;
    $current_index = 0;

    # Remove the whitespace at the beginning and end of the lines
    foreach $line (@com_file)
    {   $line =~ s/^\s+|\s+$//g;  }

    if ($com_file [$current_index++] eq $com_div)
    {
        while (!$end_loop)
        {
            $comments [$num_comments][0] = $com_file [$current_index++];
            $comments [$num_comments][1] = $com_file [$current_index++];
            $comments [$num_comments][2] = $com_file [$current_index++];
            $comments [$num_comments][3] = $com_file [$current_index++];
            $comments [$num_comments][4] = "";
            while ($current_index <= $#com_file && $com_file [$current_index] ne $com_div)
            {
                $comments [$num_comments][4] = $comments [$num_comments][4] .
                    $com_file [$current_index++] . "\n";
            }
            $num_comments++;
            $current_index++;
            if ($current_index > $#com_file)
            {   $end_loop = 1;  }
        }
    }
    else
    {   print_read_error;   }
}

# Prints one single comment
sub print_individual_comment
{
    $name  = $_ [0]->[0];
    $email = $_ [0]->[1];
    $url   = $_ [0]->[2];
    $date  = $_ [0]->[3];
    $msg   = $_ [0]->[4];
    
    @date  = split (/ /, $date);

    $year  = $date [0];
    $month = $date [1];
    $day   = $date [2];
    $hour  = $date [3];
    $min   = $date [4];

    print "<div class=\"comment\">\n";
    print "<div class=\"comment_text\">\n";
    @msg = split ("\n", $msg);
    foreach $line (@msg)
    {   print p ($line), "\n";  }
    print "<\/div>\n<div class=\"comment_data\">\n";
    print "<p>$name | ";
    if ($email ne "")
    {   print "<a href=\"mailto://$email\">Email</a> | ";    }
    else
    {   print "Email | ";    }
    if ($url ne "")
    {   
        if (substr ($url, 0, 7) ne "http://")
        {   print "<a href=\"http://$url\">Web Page</a> | ";   }
        else
        {   print "<a href=\"$url\">Web Page</a> | ";   }
    }
    else
    {   print "Web Page | ";   }
    print "$hour:$min $day\/$month\/$year UTC</p>";
    print "</div>\n</div>\n";
}

# Prints the comments
sub print_comments
{
    # Read the comments into an array of arrays
    read_comments;

    for ($i = 0; $i < $num_comments; $i++)
    {   print_individual_comment ($comments [$i]); }
}

# Prints the form to add a new comment
sub print_add_form
{
    print "<div id=\"comments_submit\">\n";
    print start_form (-onSubmit=>"if
    (document.forms.comment_submit.name.value == '' ||
     document.forms.comment_submit.name.value == null ||
     document.forms.comment_submit.msg.value  == '' ||
     document.forms.comment_submit.msg.value  == null)
    {alert('Both name and comment are required');return false;}
    else
    {return true;}", -name=>"comment_submit"), "\n";
    print "<table style=\"border-width:0;\">\n";
    print Tr (td ("Name:") .      td (textfield ("name",  "", 50, 100))), "\n";
    print Tr (td ("Email:") .     td (textfield ("email", "", 50, 100))), "\n";
    print Tr (td ("Home Page:") . td (textfield ("url",   "", 50, 100))), "\n";
    param ("msg", ""); # Make sure the message box is empty
    print Tr (td ("Comment:") .   td (textarea  ("msg",   "", 10, 50))), "\n";
    print "<\/table>";
    print hidden ("action", "add"), "\n";
    print hidden ("post", $post), "\n";
    print submit (-value=>"Add Comment"), "\n";
    print end_form, "\n";
    print "<\/div>\n";
    print p ({-id=>'comments_instruction'}, "Email and Home Page are both 
        optional. All HTML will be escaped.");
}

# Run if the user wishes to view comments for a particular article
sub view_comments
{
    $post = get_post;
    print start_html (-title=>"$page_name Comments", -style=>{'src'=>$css});
    print_top_of_page ($post);
    if (!(-s "..\/$post\/comments"))
    {
        print h1 ({-id=>"no_comments"}, "No Comments");
        print p ("No one has commented on this post yet, <i>you<\/i> can be the first!!!!");
    }
    else
    {   print_comments; }

    print_add_form;
    print end_html;
}

# Adding a comment
sub add_comment
{
    $post = get_post;

    $name  = param ("name");
    $email = param ("email");
    $url   = param ("url");
    $msg   = param ("msg");

    # Make sure the user hasn't inputed something that is the same as the
    # $com_div
    if ($name eq $com_div)
    {   $name = $name . ".";    }
    if ($email eq $com_div)
    {   $email = $email . ".";    }
    if ($url eq $com_div)
    {   $url = $url . ".";    }
    @msg = split ("\n", $msg);
    foreach $line (@msg)
    {   if ($line eq $com_div)  {   $line = $line . ".";    }   }
    $msg = join ("\n", @msg);

    @localtime = localtime ();
    $date = "";
    $date = $date .       ($localtime [5] + 1900);
    $date = $date . " " . ($localtime [4] + 1);
    $date = $date . " " .  $localtime [3];
    $date = $date . " " .  $localtime [2];
    $date = $date . " " .  $localtime [1];

    open (COMFILE, ">>..\/$post\/comments") || print_read_error && return; 

    print COMFILE HTML::Entities::encode ($com_div .  "\n" .  $name .  "\n" .  
            $email .  "\n" .  $url .  "\n" .  $date .  "\n" .  $msg .  "\n");

    close (COMFILE);

    # View the comments after adding the new one
    view_comments;
}

# The main function of the program
sub main
{
    # Get the configuration values
    get_conf;

    # Print the header
    print header;

    # If something was passed to the script
    if (param ())
    {
        if (param ("action") eq "add")
        {   add_comment;  }
        else
        {   view_comments;   }
    }

    # If nothing was passed to the script
    else
    {   $error_msg = "No Parameters passed"; print_query_error; }

    # Print the end of the file
    print end_html;
}

# Execute the mainline
main;
